/**********************************************************************

This file is part of the Quantum Computation Library.

Copyright by Bernhard Oemer 1996, all rights reserved

This program comes without any warranty; without even the implied 
warranty of merchantability or fitness for any particular purpose.

The use and distribution in any electronic or non-electronic way
of this or any other file of this package is forbidden without 
prior written or PGP-signed permission from the author.

E-mail: oemer@tph.tuwien.ac.at

PGP fingerprint:     51 BB 4A CE CB 15 32 DE  A7 64 BC F4 05 F1 35 A8
PGP public key:      http://tph.tuwien.ac.at/~oemer/pubkey.txt

************************************************************************/

#ifndef _LIBQC_H
#define _LIBQC_H 1

#include "bitvec.h"
#include "terms.h"
#include "qustates.h"
#include "operator.h"

#endif

